import pytest
from pathlib import Path
from blackfennec.type_system.string_type import StringType
from blackfennec.type_system.type_loader import TypeLoader
from blackfennec.structure.string import String
from blackfennec.type_system.type_registry import TypeRegistry

from blackfennec.document_system.document_factory import DocumentFactory
from blackfennec.document_system.mime_type.mime_type_registry import (
    MimeTypeRegistry,
)
from blackfennec.document_system.mime_type.in_memory.in_memory_mime_type import (
    InMemoryMimeType,
)
from blackfennec.document_system.mime_type.json.json_mime_type import (
    JsonMimeType,
)
from blackfennec.document_system.mime_type.json.json_pointer_serializer import (
    JsonPointerSerializer,
)
from blackfennec.document_system.mime_type.json.json_reference_serializer import (
    JsonReferenceSerializer,
)
from blackfennec.structure.structure_serializer import (
    StructureSerializer,
)
from blackfennec.document_system.resource_type.protocols.bftype_resource_type import (
    BFTypeResourceType,
)
from blackfennec.document_system.resource_type.protocols.file_resource_type import (
    FileResourceType,
)
from blackfennec.document_system.resource_type.resource_type_registry import (
    ResourceTypeRegistry,
)
from blackfennec_doubles.document_system.double_document_registry import DocumentRegistryMock


BASE_NAME = Path(__file__).parent.parent.parent.as_posix()


@pytest.fixture
def type_registry():
    type_registry = TypeRegistry()
    type_registry.register_type(StringType())

    return type_registry


@pytest.fixture
def document_factory(type_registry) -> DocumentFactory:
    resource_type_registry = ResourceTypeRegistry()
    resource_types = [
        FileResourceType(),
        BFTypeResourceType(type_registry),
    ]
    for resource_type in resource_types:
        for protocol in resource_type.protocols:
            resource_type_registry.register_resource_type(
                protocol, resource_type)

    mime_type_registry = MimeTypeRegistry()
    document_factory = DocumentFactory(
        DocumentRegistryMock(), resource_type_registry, mime_type_registry)
    reference_parser = JsonReferenceSerializer(
        document_factory, JsonPointerSerializer)
    structure_serializer = StructureSerializer(reference_parser)
    mime_types = [
        JsonMimeType(structure_serializer),
        InMemoryMimeType(),
    ]
    for mime_type in mime_types:
        mime_type_registry.register_mime_type(
            mime_type.mime_type_id, mime_type)
    return document_factory


@pytest.fixture
def type_loader(document_factory, type_registry):
    return TypeLoader(document_factory, type_registry)


@pytest.fixture
def type(type_loader):
    return type_loader.load(BASE_NAME + "/base/url/url.json")


def test_can_load(type):
    assert type is not None


def test_name_is_url(type):
    assert type.name == "Url"


def test_super_type_is_string(type):
    assert type.super.name == "String"


@pytest.mark.parametrize(
    "structure",
    [
        String("https://blackfennec.org"),
        String("http://blackfennec.org"),
        String("blackfennec.org"),
        String("blackfennec.org/"),
        String("blackfennec.org/news")
    ]
)
def test_can_cover_url(type, structure):
    assert type.calculate_coverage(structure).is_covered()


@pytest.mark.parametrize(
    "structure",
    [
        String("https:/blackfennec"),
        String("Black Fennec is great"),
        
    ]
)
def test_can_not_cover_url(type, structure):
    assert not type.calculate_coverage(structure).is_covered()
